
;Console Input With Prompt and Filtering - Include File - DednDave 7, 2011

;#############################################################################################################

ConIn   PROTO   :LPSTR,:LPSTR,:UINT,:UINT

;*************************************************************************************************************

CI_NOFILTER         EQU 0
CI_NUMERIC          EQU 1
CI_ALPHA            EQU 2
CI_ALPHANUMERIC     EQU 3
CI_NOSPACES         EQU 4

;#############################################################################################################

        .CODE

ConIn   PROC    uses edi lpszPrompt:LPSTR,lpBuffer:LPSTR,uSize:UINT,dwMode:UINT

;Console Input With Prompt - DednDave 7, 2011

;This routine supports backspace and alpha/numeric filtering.
;Tab, line editing with arrow keys, insert and overwrite modes are not included in this preliminary version.
;
;Call With: lpszPrompt = address of prompt string
;                        if NULL, no prompt is displayed
;             lpBuffer = address of output buffer, see notes for uSize
;                uSize = size of buffer in bytes, less one byte for a terminator
;                        i.e., if the buffer is 256 bytes, use 255 as the uSize parameter
;               dwMode = alpha/numeric filter mode control
;                        0 = all characters allowed
;                        1 = numbers (0-9) and spaces allowed
;                        2 = letters (a-z, A-Z) and spaces allowed
;                        3 = letters and numbers (0-9, a-z, A-Z) and spaces allowed
;                        4 = all characters except spaces allowed
;                        5 = numbers only
;                        6 = letters only
;                        7 = letters and numbers only
;
;  Returns:        EAX = length of user-entered string
;                  ECX = length of buffer (less one, same as the uSize parameter)
;                  EDX = address of output buffer
;
; Examples:
;       INVOKE  ConIn,offset szNoFilter,offset ConBuf,sizeof ConBuf-1,CI_NOFILTER
;       INVOKE  ConIn,offset szNumOnly,offset ConBuf,sizeof ConBuf-1,CI_NUMERIC
;       INVOKE  ConIn,offset szAlphaOnly,offset ConBuf,sizeof ConBuf-1,CI_ALPHA
;       INVOKE  ConIn,offset szAlphaNumOnly,offset ConBuf,sizeof ConBuf-1,CI_ALPHANUMERIC
;       INVOKE  ConIn,offset szNoSpaces,offset ConBuf,sizeof ConBuf-1,CI_NOSPACES
;       INVOKE  ConIn,offset szNumOnly,offset ConBuf,sizeof ConBuf-1,CI_NUMERIC or CI_NOSPACES
;       INVOKE  ConIn,offset szAlphaOnly,offset ConBuf,sizeof ConBuf-1,CI_ALPHA or CI_NOSPACES
;       INVOKE  ConIn,offset szAlphaNumOnly,offset ConBuf,sizeof ConBuf-1,CI_ALPHANUMERIC or CI_NOSPACES

        LOCAL   hConOut:HANDLE

        INVOKE  GetStdHandle,STD_OUTPUT_HANDLE
        mov     hConOut,eax
        mov     edi,lpszPrompt
        or      edi,edi
        jz      ConIn0

        mov     al,0
        or      ecx,-1
        mov     edx,edi
        push    ecx
        repnz   scasb
        pop     ecx
        add     ecx,edi
        sub     ecx,edx
        call    WrCon

ConIn0: mov     eax,uSize
        mov     edi,lpBuffer       ;lpBuffer points to first buffer char
        push    eax
        add     eax,edi
        mov     uSize,eax          ;uSize points to end of buffer
        jmp short ConInA

ConIn1: cmp     al,0               ;function key - ignore
        jz      ConInA

        cmp     al,0E0h            ;extended key - ignore
        jz      ConInA

        cmp     al,8               ;backspace
        jnz     ConIn2

        cmp     edi,lpBuffer
        jbe     ConIn7

        inc     ecx
        dec     edi
        mov     eax,82008h         ;backspace, space, backspace
        inc     ecx
        jmp short ConIn9

ConIn2: test byte ptr dwMode,CI_NOSPACES
        jz      ConIn3

        cmp     al,20h
        jz      ConInA

ConIn3: cmp     al,20h
        jz      ConIn6

        test byte ptr dwMode,CI_NUMERIC
        jz      ConIn4

        cmp     al,'0'
        jb      ConIn4

        cmp     al,'9'
        jbe     ConIn6

ConIn4: test byte ptr dwMode,CI_ALPHA
        jz      ConIn5

        mov     edx,eax
        and     dl,0DFh             ;force upper case
        cmp     dl,'A'
        jb      ConIn5

        cmp     dl,'Z'
        jbe     ConIn6

ConIn5: test byte ptr dwMode,CI_ALPHANUMERIC
        jnz     ConInA

ConIn6: cmp     edi,uSize
        jb      ConIn8

ConIn7: INVOKE  Beep,750,40
        jmp short ConInA

ConIn8: stosb

ConIn9: push    eax
        mov     edx,esp
        call    WrCon
        pop     eax

ConInA: call    InKyb
        jnz     ConInB

        INVOKE  Sleep,40
        jmp     ConInA

ConInB: push    1
        cmp     al,0Dh              ;user pressed Enter
        pop     ecx
        jnz     ConIn1

        mov     ah,0Ah
        push    edi
        push    eax
        inc     ecx
        mov     edx,esp
        mov     al,ch
        stosb
        call    WrCon
        pop     ecx
        pop     eax
        mov     edx,lpBuffer
        pop     ecx
        sub     eax,edx
        ret

;*************************************************************************************************************

;Write String to Console - DednDave 7, 2011
;
;Call With: ECX = number of bytes to write
;           EDX = pointer to string
;
;  Returns: EAX = WriteFile status (0 = error, 1 = success)
;           ECX = number of bytes written
;
;All other registers preserved.

WrCon:  push    edx
        push    ecx
        mov     eax,esp
        INVOKE  WriteFile,hConOut,edx,ecx,eax,NULL
        pop     ecx
        pop     edx
        retn

ConIn   ENDP

;#############################################################################################################

InKyb   PROC

;Polled Keyboard Input - DednDave 8, 2010
;
;This function returns a keystroke in EAX if there is one in the buffer.
;If the buffer is empty, the function returns immediately.
;
;If the keyboard buffer is empty, AH = 0, AL = 0, ZF = 1.
;If the stroke is a regular key, AH = 0, AL = key char, ZF = 0.
;If the stroke is an extended key, AH = extended key, AL = E0h, ZF = 0.
;If the stroke is a function key, AH = function key, AL = 0, ZF = 0.
;
;ECX, EDX are not preserved.

        call    crt__kbhit
        or      eax,eax
        jz      InKyb1

        call    crt__getch
        and     eax,0FFh
        jz      InKyb0

        cmp     al,0E0h
        jnz     InKyb1

InKyb0: push    eax
        call    crt__getch
        pop     edx
        shl     eax,8
        or      eax,edx

InKyb1: retn

InKyb   ENDP

;#############################################################################################################

;End of Include File

